## File "PloneInstaller/Scripts/factory_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/types.py" to setup the portal types.

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def factory_install(self, out, project_name, globals):
    try:
      types_module = __import__("Configs.factory",
                                            globals, locals(),
                                           [
                                              'FACTORY_TYPES',
                                            ]
                                          )

      out.openSection("Configs.factory setup")

      # Factory Types configuration
      factory_types = getattr(types_module, 'FACTORY_TYPES', None)
      if factory_types != None:
        _installFactoryTypes(self, out, factory_types)

      out.closeSection("Configs.factory setup")

    except ImportError:
        out.info("Configs.factory setup : Noting to be done.")



##---------------------------------------------------------------------------



def factory_remove(self, out, project_name, globals):
    try:
      types_module = __import__("Configs.factory",
                                            globals, locals(),
                                           [
                                              'FACTORY_TYPES',
                                            ]
                                          )

      out.openSection("Configs.factory removal")

      # Factory Types configuration
      factory_types = getattr(types_module, 'FACTORY_TYPES', None)
      if factory_types != None:
        _removeFactoryTypes(self, out, factory_types)

      out.closeSection("Configs.factory removal")

    except ImportError:
        out.info("Configs.factory removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installFactoryTypes
# 
# Setup factory types
def _installFactoryTypes(self, out, factory_types):
    errorFlag = False

    out.openSubsection("Factory types setup")

    try: # Check for factory tool existance
        factory_tool = getToolByName(self, 'portal_factory')
    except AttributeError:
        out.error("'portal_factory' tool not found !")
    else:
        for type_name in factory_types:
          try:
              existing_factory_types = factory_tool.getFactoryTypes().keys()
              if type_name not in existing_factory_types:
                  existing_factory_types.append(type_name)
                  factory_tool.manage_setPortalFactoryTypes(listOfTypeIds = existing_factory_types)
                  out.log("Registered type '%s' into portal factory." % type_name)
              else:
                  out.log("Type '%s' is already registered into portal factory." % type_name)
          except Exception, e:
              errorFlag = True
              out.error("failed to register type '%s' into factory. (Error message : %s)" % (type_name, str(e),) )

        if not errorFlag:
            out.success("Completed factory types setup without errors.")

    out.closeSubsection("Factory types setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeFactoryTypes
# 
# Remove factory types configuration.
def _removeFactoryTypes(self, out, factory_types):
    errorFlag = False

    out.openSubsection("Factory types removal")

    try: # Check for factory tool existance
        factory_tool = getToolByName(self, 'portal_factory')
    except AttributeError:
        out.error("'portal_factory' tool not found !")
    else:
        for type_name in factory_types:
          try:
              existing_factory_types = factory_tool.getFactoryTypes().keys()
              if type_name in existing_factory_types:
                  existing_factory_types.remove(type_name)
                  factory_tool.manage_setPortalFactoryTypes(listOfTypeIds = existing_factory_types)
                  out.log("Unregistered type '%s' into portal factory." % type_name)
              else:
                  out.log("Type '%s' is not registered into portal factory." % type_name)
          except Exception, e:
              errorFlag = True
              out.error("failed to unregister type '%s' into factory. (Error message : %s)" % (type_name, str(e),) )

        if not errorFlag:
            out.success("Completed factory types removal without errors.")

    out.closeSubsection("Factory types removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

