## File "PloneInstaller/Scripts/objects_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/objects.py" to setup the portal objects.

from Products.CMFCore.utils import getToolByName

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def objects_install(self, out, project_name, globals):
    try:
      objects_module = __import__("Configs.objects",
                                            globals, locals(),
                                           [
                                              'CREATED_OBJECTS',
                                              'UPDATED_OBJECTS',
                                              'DELETED_OBJECTS',
                                            ]
                                          )

      out.openSection("Configs.objects setup")

      # Created objects configuration
      created_objects = getattr(objects_module, 'CREATED_OBJECTS', None)
      if created_objects != None:
        _installCreatedObjects(self, out, created_objects)

      # Updated objects configuration
      updated_objects = getattr(objects_module, 'UPDATED_OBJECTS', None)
      if updated_objects != None:
        _installUpdatedObjects(self, out, updated_objects)

      # Deleted objects configuration
      deleted_objects = getattr(objects_module, 'DELETED_OBJECTS', None)
      if deleted_objects != None:
        _installDeletedObjects(self, out, deleted_objects)

      out.closeSection("Configs.objects setup")

    except ImportError:
        out.info("Configs.objects setup : Noting to be done.")



##---------------------------------------------------------------------------



def objects_remove(self, out, project_name, globals):
    try:
      objects_module = __import__("Configs.objects",
                                            globals, locals(),
                                           [
                                              'CREATED_OBJECTS',
                                              'UPDATED_OBJECTS',
                                              'DELETED_OBJECTS',
                                            ]
                                          )

      out.openSection("Configs.objects removal")

      # Created objects configuration
      created_objects = getattr(objects_module, 'CREATED_OBJECTS', None)
      if created_objects != None:
        _removeCreatedObjects(self, out, created_objects)

      # Updated objects configuration
      updated_objects = getattr(objects_module, 'UPDATED_OBJECTS', None)
      if updated_objects != None:
        _removeUpdatedObjects(self, out, updated_objects)

      # Deleted objects configuration
      deleted_objects = getattr(objects_module, 'DELETED_OBJECTS', None)
      if deleted_objects != None:
        _removeDeletedObjects(self, out, deleted_objects)

      out.closeSection("Configs.objects removal")

    except ImportError:
        out.info("Configs.objects removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installCreatedObjects
# 
# Create the product objects.
def _installCreatedObjects(self, out, created_objects):
    errorFlag = False

    out.openSubsection("Created objects setup")

    try: # Check for factory tool existance
      url_tool = getToolByName(self, 'portal_url')
      portal_object = url_tool.getPortalObject()
    except AttributeError:
      errorFlag = True
      out.error("'portal_url' tool not found !")
    else:
      for created_object_definition in created_objects:
        created_object_path = created_object_definition['id']

        # We check that 'created_object_definition' has the required keys.
        try:
          if not isinstance(created_object_definition, dict):
            raise Exception, "object definition is not a dictionnary."

          if not created_object_definition.has_key("type"):
            raise Exception, "object definition has no key 'type'."

          if not isinstance(created_object_definition["type"], str):
            raise Exception, "object definition 'type' attribute is not a string."

          if not created_object_definition.has_key("parameters"):
            raise Exception, "object definition has no key 'parameters'."

          if not isinstance(created_object_definition["parameters"], dict):
            raise Exception, "object definition 'parameters' attribute is not a dictionnary."
        except Exception, e:
          errorFlag = True
          out.error("Error in '%s' object definition : %s" % (created_object_path, str(e)))
        else:
          # We explode the path to get containers id.
          created_object_exploded_path = created_object_path.split('/')

          # We get the created object id.
          created_object_id = created_object_exploded_path.pop()

          # The root container is the portal.
          container = portal_object

          # We run through the path to get the container of the object to be created.
          for object_id in created_object_exploded_path:
            # Check if the object with id 'object_id' exist into the current container.
            if not object_id in container.objectIds():
              container = None
              break;
            # If it exist, it become the current container.
            container = getattr(container, object_id, None)
            # We stop if the object is not found.
            if not container:
              break;

          # Whe check if the object to be created is not already present.
          if (not container) or (created_object_id in container.objectIds()):
            out.info("Object '%s' exist." % created_object_path)
          else:
            # Create object by invoking factory.
            created_id = container.invokeFactory(id = created_object_id, type_name = created_object_definition['type'])
            if (created_id is None) or (created_id == ''):
              created_id = created_object_id
            created_object = getattr(container, created_id, None)
            # Check for object creation.
            if not created_object:
              out.error("Failed to create object '%s'." % created_object_path)
              errorFlag = True
            else:
              # The new object has been successfuly created.
              # Now we configure it.
              created_object.edit(**created_object_definition['parameters'])

              # Set the navigation exclusion
              if created_object_definition.has_key('exclude_from_nav') :
                if getattr(created_object, "update", None) :
                  created_object.update(excludeFromNav = created_object_definition['exclude_from_nav'])
                else:
                  created_object.exclude_from_nav = created_object_definition['exclude_from_nav']

              # Set object properties
              if created_object_definition.has_key('properties') :
                for property in created_object_definition['properties']:
                  if(created_object.hasProperty(property['id'])):
                    created_object._updateProperty(property['id'], property['value'])
                  else:
                    created_object.manage_addProperty(**property)

              # Delete object properties
              if created_object_definition.has_key('deleted_properties') :
                created_object.manage_delProperty(created_object_definition['deleted_properties'])

              # Set object allowed contents
              if created_object_definition.has_key('constrain_types_mode') :
                created_object.setConstrainTypesMode(created_object_definition['constrain_types_mode'])
              if created_object_definition.has_key('locally_allowed_types') :
                created_object.setLocallyAllowedTypes(created_object_definition['locally_allowed_types'])
              if created_object_definition.has_key('immediately_addable_types') :
                created_object.setImmediatelyAddableTypes(created_object_definition['immediately_addable_types'])

              out.info("Object '%s' successfully created." % created_object_path)

      if not errorFlag:
        out.success("Completed created objects setup without errors.")

    out.closeSubsection("Created objects setup")



##---------------------------------------------------------------------------



## _installUpdatedObjects
# 
# Update the portal objects.
def _installUpdatedObjects(self, out, updated_objects):
    errorFlag = False

    out.openSubsection("Updated objects setup")

    try: # Check for factory tool existance
      url_tool = getToolByName(self, 'portal_url')
      portal_object = url_tool.getPortalObject()
    except AttributeError:
      errorFlag = True
      out.error("'portal_url' tool not found !")
    else:
      for updated_object_definition in updated_objects:
        updated_object_path = updated_object_definition['id']

        # We check that 'updated_object_definition' has the required keys.
        try:
          if not isinstance(updated_object_definition, dict):
            raise Exception, "object definition is not a dictionnary."
        except Exception, e:
          errorFlag = True
          out.error("Error in '%s' object definition : %s" % (updated_object_path, str(e)))
        else:
          # We explode the path to get containers id.
          updated_object_exploded_path = updated_object_path.split('/')

          # We get the updated object id.
          updated_object_id = updated_object_exploded_path.pop()

          # The root container is the portal.
          container = portal_object

          if updated_object_path != '': 
            # We run through the path to get the container of the object to be updated.
            for object_id in updated_object_exploded_path:
              # Check if the object with id 'object_id' exist into the current container.
              if not object_id in container.objectIds():
                container = None
                break;
              # If it exist, it become the current container.
              container = getattr(container, object_id, None)
              # We stop if the object is not found.
              if not container:
                break;

          # Whe check if the object to be updated exist.
          if not (container and ((updated_object_id in container.objectIds()) or (updated_object_path == ''))):
            out.error("Object '%s' does not exist." % updated_object_path)
            errorFlag = True
          else:
            if updated_object_path == '' :
              updated_object = container
            else:
              updated_object = getattr(container, updated_object_id, None)
            # Check for object creation.
            if not updated_object:
              out.error("Failed to update object '%s'." % updated_object_path)
              errorFlag = True
            else:
              # The object to be updated is found.
              # Now we configure it.

              # Set the navigation exclusion
              if updated_object_definition.has_key('exclude_from_nav') :
                if getattr(updated_object, "update", None) :
                  updated_object.update(excludeFromNav = updated_object_definition['exclude_from_nav'])
                else:
                  updated_object.exclude_from_nav = updated_object_definition['exclude_from_nav']

              # Set object properties
              if updated_object_definition.has_key('properties') :
                for property in updated_object_definition['properties']:
                  if(updated_object.hasProperty(property['id'])):
                    updated_object._updateProperty(property['id'], property['value'])
                  else:
                    updated_object.manage_addProperty(**property)

              # Delete object properties
              if updated_object_definition.has_key('deleted_properties') :
                updated_object.manage_delProperty(updated_object_definition['deleted_properties'])

              # Set object allowed contents
              if updated_object_definition.has_key('constrain_types_mode') :
                updated_object.setConstrainTypesMode(updated_object_definition['constrain_types_mode'])
              if updated_object_definition.has_key('locally_allowed_types') :
                updated_object.setLocallyAllowedTypes(updated_object_definition['locally_allowed_types'])
              if updated_object_definition.has_key('immediately_addable_types') :
                updated_object.setImmediatelyAddableTypes(updated_object_definition['immediately_addable_types'])

              out.info("Object '%s' successfully updated." % updated_object_path)

      if not errorFlag:
        out.success("Completed updated objects setup without errors.")

    out.closeSubsection("Updated objects setup")



##---------------------------------------------------------------------------



## _installDeletedObjects
# 
# Delete portal objects.
def _installDeletedObjects(self, out, deleted_objects):
    errorFlag = False

    out.openSubsection("Deleted objects setup")

    try: # Check for factory tool existance
      url_tool = getToolByName(self, 'portal_url')
      portal_object = url_tool.getPortalObject()
    except AttributeError:
      errorFlag = True
      out.error("'portal_url' tool not found !")
    else:
      for deleted_object_path in deleted_objects:
        # We explode the path to get containers id.
        deleted_object_exploded_path = deleted_object_path.split('/')

        # We get the deleted object id.
        deleted_object_id = deleted_object_exploded_path.pop()

        # The root container is the portal.
        container = portal_object

        # We run through the path to get the container of the object to be deleted.
        for object_id in deleted_object_exploded_path:
          # Check if the object with id 'object_id' exist into the current container.
          if not object_id in container.objectIds():
            container = None
            break;
          # If it exist, it become the current container.
          container = getattr(container, object_id, None)
          # We stop if the object is not found.
          if not container:
            break;

        # Whe check if the object to be deleted exist.
        if not (container and (deleted_object_id in container.objectIds())):
          out.info("Object '%s' does not exist." % deleted_object_path)
        else:
          container._delObject(deleted_object_id)
          # Check for object deletion.
          if deleted_object_id in container.objectIds():
            out.error("Failed to delete object '%s'." % updated_object_path)
            errorFlag = True
          else:
            out.info("Object '%s' successfully deleted." % deleted_object_path)

      if not errorFlag:
        out.success("Completed deleted objects setup without errors.")

    out.closeSubsection("Deleted objects setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeCreatedObjects
# 
# Remove objects created by this product installation.
# TODO : Create a remove created objects method.
def _removeCreatedObjects(self, out, created_objects):

    out.openSubsection("Created objects removal")

    try: # Check for types tool existance
        url_tool = getToolByName(self, 'portal_url')
    except AttributeError:
        out.error("'portal_url' tool not found !")
    else:
        out.info("Created objects configuration can not be removed.")

    out.closeSubsection("Created objects removal")



##---------------------------------------------------------------------------



## _removeUpdatedObjects
# 
# Restore objects updated by this product installation.
# TODO : Create a remove updated objects method.
def _removeUpdatedObjects(self, out, updated_objects):

    out.openSubsection("Updated objects removal")

    try: # Check for types tool existance
        url_tool = getToolByName(self, 'portal_url')
    except AttributeError:
        out.error("'portal_url' tool not found !")
    else:
        out.info("Updated objects configuration can not be removed.")

    out.closeSubsection("Updated objects removal")



##---------------------------------------------------------------------------



## _removeDeletedObjects
# 
# Restore objects deleted by this product installation.
# TODO : Create a remove deleted objects method.
def _removeDeletedObjects(self, out, deleted_objects):

    out.openSubsection("Deleted objects removal")

    try: # Check for types tool existance
        url_tool = getToolByName(self, 'portal_url')
    except AttributeError:
        out.error("'portal_url' tool not found !")
    else:
        out.info("Deleted objects configuration can not be removed.")

    out.closeSubsection("Deleted objects removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

