## File "PloneInstaller/Scripts/placeful_workflows_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/placeful_workflows.py" to setup the portal placeful workflows.



from Products.CMFCore.utils import getToolByName



#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def placeful_workflows_install(self, out, project_name, globals):
    try:
      security_module = __import__("Configs.placeful_workflows",
                                            globals, locals(),
                                           [
                                              'ADDED_PLACEFUL_WORKFLOWS',
                                              'PLACEFUL_WORKFLOWS_POLICIES',
                                            ]
                                          )

      out.openSection("Configs.placeful_workflows setup")

      # Added placeful workflows configuration
      added_placeful_workflows = getattr(security_module, 'ADDED_PLACEFUL_WORKFLOWS', None)
      if added_placeful_workflows != None:
        _installAddedPlacefulWorkflows(self, out, added_placeful_workflows)

      # placeful workflows policies configuration
      placeful_workflows_policies = getattr(security_module, 'PLACEFUL_WORKFLOWS_POLICIES', None)
      if placeful_workflows_policies != None:
        _installPlacefulWorkflowsPolicies(self, out, placeful_workflows_policies)

      out.closeSection("Configs.placeful_workflows setup")

    except ImportError:
        out.info("Configs.placeful_workflows setup : Noting to be done.")



##---------------------------------------------------------------------------



def placeful_workflows_remove(self, out, project_name, globals):
    try:
      security_module = __import__("Configs.placeful_workflows",
                                            globals, locals(),
                                           [
                                              'ADDED_PLACEFUL_WORKFLOWS',
                                              'PLACEFUL_WORKFLOWS_POLICIES',
                                            ]
                                          )

      out.openSection("Configs.placeful_workflows removal")

      # placeful workflows policies configuration
      placeful_workflows_policies = getattr(security_module, 'PLACEFUL_WORKFLOWS_POLICIES', None)
      if placeful_workflows_policies != None:
        _removePlacefulWorkflowsPolicies(self, out, placeful_workflows_policies)

      # Added placeful workflows configuration
      added_placeful_workflows = getattr(security_module, 'ADDED_PLACEFUL_WORKFLOWS', None)
      if added_placeful_workflows != None:
        _removeAddedPlacefulWorkflows(self, out, added_placeful_workflows)

      out.closeSection("Configs.placeful_workflows removal")

    except ImportError:
        out.info("Configs.placeful_workflows removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installAddedPlacefulWorkflows
# 
# Setup added roles configuration
def _installAddedPlacefulWorkflows(self, out, added_placeful_workflows):
    errorFlag = False

    out.openSubsection("Added placeful workflows setup")

    try: # Check for tool existance
      placeful_workflow_tool = getToolByName(self, 'portal_placeful_workflow')
    except AttributeError:
      out.error("'portal_placeful_workflow' tool not found !")
    else:
      for placeful_workflow_id, placeful_workflow_definition in added_placeful_workflows.items():
        try:
          placeful_workflow_tool.manage_addWorkflowPolicy(id = placeful_workflow_id)
        except Exception, e:
          errorFlag = True
          out.error("Unable to add placeful workflow policy '%s'. (error message : %s)" % (placeful_workflow_id, str(e)) )
        else:
          policy = placeful_workflow_tool.getWorkflowPolicyById(placeful_workflow_id)
          if placeful_workflow_definition.has_key('title'):
            policy.setTitle(placeful_workflow_definition['title'])
          if placeful_workflow_definition.has_key('description'):
            policy.setDescription(placeful_workflow_definition['description'])

          if placeful_workflow_definition.has_key('types'):
            for type_name, type_workflow in placeful_workflow_definition['types'].items():
              policy.setChain(portal_type=type_name, chain=(type_workflow,))

          if placeful_workflow_definition.has_key('default'):
            policy.setDefaultChain(default_chain=(placeful_workflow_definition['default'],))

          out.info("Added placeful workflow policy '%s'." % placeful_workflow_id)

      if not errorFlag:
        out.success("Completed added placeful workflows setup without errors.")

    out.closeSubsection("Added placeful workflows setup")



## _installPlacefulWorkflowsPolicies
# 
# Setup added roles configuration
def _installPlacefulWorkflowsPolicies(self, out, placeful_workflows_policies):
    errorFlag = False

    out.openSubsection("Placeful workflows policies setup")

    try: # Check for tool existance
      placeful_workflow_tool = getToolByName(self, 'portal_placeful_workflow')
      url_tool = getToolByName(self, 'portal_url')
      portal_object = url_tool.getPortalObject()
    except AttributeError:
      out.error("'portal_placeful_workflow' tool not found !")
    else:
      for updated_object_path, updated_object_workflow_policy in placeful_workflows_policies.items():
        # We explode the path to get containers id.
        updated_object_exploded_path = updated_object_path.split('/')

        # We get the updated object id.
        updated_object_id = updated_object_exploded_path.pop()

        # The root container is the portal.
        container = portal_object

        if updated_object_path != '': 
          # We run through the path to get the container of the object to be updated.
          for object_id in updated_object_exploded_path:
            # Check if the object with id 'object_id' exist into the current container.
            if not object_id in container.objectIds():
              container = None
              break;
            # If it exist, it become the current container.
            container = getattr(container, object_id, None)
            # We stop if the object is not found.
            if not container:
              break;

        # Whe check if the object to be updated exist.
        if not (container and ((updated_object_id in container.objectIds()) or (updated_object_path == ''))):
          out.error("Object '%s' does not exist." % updated_object_path)
          errorFlag = True
        else:
          if updated_object_path == '' :
            updated_object = container
          else:
            updated_object = getattr(container, updated_object_id, None)
          # Check for object creation.
          if not updated_object:
            out.error("Failed to update object '%s'." % updated_object_path)
            errorFlag = True
          else:
            # The object to be updated is found.
            # Now we configure it.
            updated_object_policy = placeful_workflow_tool.getWorkflowPolicyConfig(updated_object)
            if not updated_object_policy:
              updated_object.manage_addProduct['CMFPlacefulWorkflow'].manage_addWorkflowPolicyConfig()
              updated_object_policy = placeful_workflow_tool.getWorkflowPolicyConfig(updated_object)

            if(updated_object_workflow_policy.has_key('in')) :
              policy_in = updated_object_workflow_policy['in']
            else:
              policy_in = ""

            if placeful_workflow_tool.getWorkflowPolicyById(policy_in):
              updated_object_policy.setPolicyIn(policy=policy_in)
            elif policy_in == '':
              updated_object_policy.setPolicyIn(policy='')
            else:
              errorFlag = True
              out.error("'%s' is not a valid placeful workflow in object '%s'." % (policy_in, updated_object_path))

            if(updated_object_workflow_policy.has_key('below')) :
              policy_below = updated_object_workflow_policy['below']
            else:
              policy_below = ""

            if placeful_workflow_tool.getWorkflowPolicyById(policy_below):
              updated_object_policy.setPolicyBelow(policy=policy_below)
            elif policy_below == '':
              updated_object_policy.setPolicyBelow(policy='')
            else:
              errorFlag = True
              out.error("'%s' is not a valid placeful workflow below object '%s'." % (policy_below, updated_object_path))

            out.info("Setted up placeful workflow policy for object '%s'." % updated_object_path)

      if not errorFlag:
        out.success("Completed added placeful workflows setup without errors.")

    out.closeSubsection("Placeful workflows policies setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeAddedPlacefulWorkflows
# 
# Remove added placeful workflows configuration.
# TODO : Create a remove added placeful workflows method.
def _removeAddedPlacefulWorkflows(self, out, added_placeful_workflows):

    out.openSubsection("Added placeful workflows removal")

    try: # Check for tool existance
        placeful_workflow_tool = getToolByName(self, 'portal_placeful_workflow')
    except AttributeError:
        out.error("'portal_placeful_workflow' tool not found !")
    else:
      out.info("Added placeful workflows configuration can not be removed.")

    out.closeSubsection("Added placeful workflows removal")



## _removePlacefulWorkflowsPolicies
# 
# Remove placeful workflows policies configuration.
# TODO : Create a remove placeful workflows policies method.
def _removePlacefulWorkflowsPolicies(self, out, placeful_workflows_policies):

    out.openSubsection("Placeful workflows policies removal")

    try: # Check for tool existance
        placeful_workflow_tool = getToolByName(self, 'portal_placeful_workflow')
    except AttributeError:
        out.error("'portal_placeful_workflow' tool not found !")
    else:
      out.info("Placeful workflows policies configuration can not be removed.")

    out.closeSubsection("Placeful workflows policies removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

