## File "PloneInstaller/Scripts/workflows_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/workflows/" to setup the portal workflows.

import os
from Globals import package_home
from Products.CMFCore.utils import getToolByName
from Products.PythonScripts.PythonScript import PythonScript

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def workflows_install(self, out, project_name, globals):
    try:

      # We list files in 'Config/workflows' for import.
      workflows_path = package_home(globals) + '/Configs/workflows/'
      workflows_list = []
      if os.path.exists(workflows_path):
        file_list = os.listdir(workflows_path)
        for filesystem_item in file_list:
          if not(filesystem_item.startswith("_")) and filesystem_item.endswith(".py"):
            workflows_list.append(filesystem_item.replace(".py", ""))

      workflows_module = __import__("Configs.workflows",
                                    globals, locals(),
                                    workflows_list
                                  )

      if len(workflows_list) > 0:
        out.openSection("Configs.workflows setup")

        for workflow_id, workflow_module in workflows_module.__dict__.items():
          if not workflow_id.startswith("_"):
            # For each workflow, we install it.
            out.openSubsection("workflow setup : '%s'" % workflow_id)
            workflow_definition = workflow_module.__dict__.get("workflow_definition", None)
            _installWorkflow(self, out, globals, workflow_id, workflow_definition)
            workflow_associated_types = workflow_module.__dict__.get("workflow_associated_types", ())
            _installWorkflowAssociatedTypes(self, out, workflow_id, workflow_associated_types)
            out.closeSubsection("workflow setup : '%s'" % workflow_id)

        _updateWorkflowRoleMappings(self, out)

        out.closeSection("Configs.workflows setup")

    except ImportError:
        out.info("Configs.workflows setup : Noting to be done.")



##---------------------------------------------------------------------------



def workflows_remove(self, out, project_name, globals):
    try:
      # We list files in 'Config/workflows' for import.
      workflows_path = package_home(globals) + '/Configs/workflows/'
      workflows_list = []
      out.info(workflows_path)
      if os.path.exists(workflows_path):
        file_list = os.listdir(workflows_path)
        for filesystem_item in file_list:
          if not(filesystem_item.startswith("_")) and filesystem_item.endswith(".py"):
            workflows_list.append(filesystem_item.replace(".py", ""))

      if len(workflows_list) > 0:
        workflows_module = __import__("Configs.workflows",
                                      globals, locals(),
                                      workflows_list
                                    )

        out.openSection("Configs.workflows removal")

        for workflow_id, workflow_module in workflows_module.__dict__.items():
          if not workflow_id.startswith("_"):
            # For each workflow, we remove it.
            out.openSubsection("workflow removal : '%s'" % workflow_id)
            workflow_associated_types = workflow_module.__dict__.get("workflow_associated_types", ())
            _removeWorkflowAssociatedTypes(self, out, workflow_id, workflow_associated_types)
            workflow_definition = workflow_module.__dict__.get("workflow_definition", None)
            _removeWorkflow(self, out, workflow_id, workflow_definition)
            out.closeSubsection("workflow removal : '%s'" % workflow_id)

        out.closeSection("Configs.workflows removal")

    except ImportError:
        out.info("Configs.workflows removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installWorkflow
# 
# Create the product workflows.
def _installWorkflow(self, out, globals, workflow_id, workflow_definition):
    errorFlag = False

    try: # Check for factory tool existance
      workflow_tool = getToolByName(self, 'portal_workflow')
    except AttributeError:
      errorFlag = True
      out.error("'portal_workflow' tool not found !")
    else:
      try: # Check for workflow_definition validity
        if not workflow_definition:
          raise Exception, "'workflow_definition' for workflow '%s' is not defined." % workflow_id
        required_attributes = ( 'title', 'managedPermissions', 'transitions', 'states', 'initial_state', 'variables', 'state_variable', 'worklists', 'scripts')
        for attribute_id in required_attributes:
          if not workflow_definition.has_key(attribute_id):
            raise Exception, "Definition of workflow '%s' has no '%s' attribute." % (workflow_id, attribute_id,)
      except Exception, e:
        errorFlag = True
        out.error(str(e))
      else: # workflow_definition is OK.

        if(workflow_tool.getWorkflowById(workflow_id)):
          out.info("Replacing existing '%s' workflow." % workflow_id)

        # Worflow creation.
        workflow_tool.manage_addWorkflow(id= workflow_id,
                                         workflow_type='dc_workflow (Web-configurable workflow)')

        workflow = workflow_tool.getWorkflowById(workflow_id)

        if not workflow: # Test for workflow creation
          errorFlag = True
          out.error("'%s' workflow has not been created." % workflow_id)
        else:
          # Set workflow title
          workflow.setProperties(title=workflow_definition.get("title"))

          # Adding managed permissions
          for managedPermission_id in workflow_definition.get("managedPermissions"):
            workflow.addManagedPermission(managedPermission_id)

          # Adding transitions
          for transition_id, transition_definition in workflow_definition.get("transitions").items():
            workflow.transitions.addTransition(transition_id)
            transition = workflow.transitions[transition_id]
            transition.setProperties(**transition_definition)

          # Adding states
          for state_id, state_definition in workflow_definition.get("states").items():
            workflow.states.addState(state_id)
            state = workflow.states[state_id]

            state.setProperties(title = state_definition['title'],
                                transitions = state_definition['transitions'])
            for permission_settings in state_definition['permissions']:
              state.setPermission(**permission_settings)

          # Setting initial state.
          workflow.states.setInitialState(workflow_definition.get("initial_state"))

          # Adding variables
          for variable_id, variable_definition in workflow_definition.get("variables").items():
            workflow.variables.addVariable(variable_id)
            variable = workflow.variables[variable_id]
            variable.setProperties(**variable_definition)

          workflow.variables.setStateVar(workflow_definition.get("state_variable"))

          # Adding worklists
          for worklist_id, worklist_definition in workflow_definition.get("worklists").items():
            workflow.worklists.addWorklist(worklist_id)
            worklist = workflow.worklists[worklist_id]
            worklist.setProperties(**worklist_definition)

          # Adding scripts
          for script_id, script_file in workflow_definition.get("scripts").items():
            real_id = workflow.scripts._setObject(script_id, PythonScript(script_id))
            file = open(package_home(globals) + '/workflow_scripts/' + script_file).read()
            workflow.scripts._getOb(real_id).write(file)

          out.success("Completed workflow creation.")



##---------------------------------------------------------------------------



## _installWorkflowAssociatedTypes
# 
# Register workflow associated types for this product installation.
def _installWorkflowAssociatedTypes(self, out, workflow_id, workflow_associated_types):
    errorFlag = False

    try: # Check for types tool existance
        workflow_tool = getToolByName(self, 'portal_workflow')
    except AttributeError:
        out.error("'portal_workflow' tool not found !")
    else:
      if(not workflow_tool.getWorkflowById(workflow_id)):
        out.error("Workflow '%s' does not exists." % workflow_id)
        errorFlag = True;
      else: 
        if workflow_associated_types == '__default__':
          workflow_tool.setDefaultChain(workflow_id)
          out.info("Registered workflow '%s' as default workflow." % workflow_id)
        else:
          if len(workflow_associated_types) > 0:
            workflow_tool.setChainForPortalTypes(workflow_associated_types, workflow_id)
            out.info("Registered workflow '%s' with types : %s." % (workflow_id, ", " . join(workflow_associated_types)))

        out.success("Completed workflow associated types registration.")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeWorkflow
# 
# Remove workflow created by this product installation.
# TODO : Create a remove workflow method.
def _removeWorkflow(self, out, workflow_id, workflow_definition):

    try: # Check for types tool existance
        workflow_tool = getToolByName(self, 'portal_workflow')
    except AttributeError:
        out.error("'portal_workflow' tool not found !")
    else:
        out.info("Workflow '%s' can not be removed." % workflow_id)



##---------------------------------------------------------------------------



## _removeWorkflowAssociatedTypes
# 
# Remove workflow associated types registered by this product installation.
# TODO : Create a remove workflow associated types method.
def _removeWorkflowAssociatedTypes(self, out, workflow_id, workflow_associated_types):

    try: # Check for types tool existance
        workflow_tool = getToolByName(self, 'portal_workflow')
    except AttributeError:
        out.error("'portal_workflow' tool not found !")
    else:
        out.info("Workflow associated types '%s' can not be removed." % workflow_id)



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################



## _updateWorkflowRoleMapping
# 
# Remove workflow associated types registered by this product installation.
# TODO : Create a remove workflow associated types method.
def _updateWorkflowRoleMappings(self, out):

    out.openSubsection("workflow role mappings update")

    try: # Check for types tool existance
        workflow_tool = getToolByName(self, 'portal_workflow')
    except AttributeError:
        out.error("'portal_workflow' tool not found !")
    else:
        workflow_tool.updateRoleMappings()
        out.success("Completed workflow role mappings update.")

    out.closeSubsection("workflow role mapping update")



